# Copyright 2010 A4 Developers.  This software is licensed under the
# GNU General Public License version 3 (see the file COPYING).

import cairo
import gtk
import math
from numpy import array, dot
from numpy.linalg import inv
from a4lib.player import GtkCairoPlayer
from a4lib.region import Region


class Editor(GtkCairoPlayer):

    def __init__(self, file_name, drawing_area, treeview, force_loading=False):
        GtkCairoPlayer.__init__(self, file_name, drawing_area, force_loading)
        self.treeview = treeview
        self.model = gtk.ListStore(int, str, gtk.gdk.Pixbuf)

        self.treeview.set_model(self.model)
        self.treeview.set_text_column(1)
        self.treeview.set_pixbuf_column(2)

        self.treeview.set_columns(1)
        self.treeview.set_item_width(120)
        self.treeview.set_size_request(130, -1)

        #self.rotate_icon = RotateIcon()
        self.action_status = None

        for i in xrange(0, len(self.presentation.path)):
            region = self.presentation.get_path_region(i)
            self.draw_region_on_iconview(region)

    def on_area_button_press_event(self, widget, event):
        GtkCairoPlayer.on_area_button_press_event(self, widget, event)
        if event.state & gtk.gdk.CONTROL_MASK:
            self.action_status = "selection"
            self.selection_start_position = (event.x, event.y)
        ## TODO: implement rotateicon
        # elif self.mouse_position in RotateIcon.get_bbox():
        #   self.action_status = rotation

    def on_area_button_release_event(self, widget, event):
        GtkCairoPlayer.on_area_button_release_event(self, widget, event)
        if self.action_status is not None:
            rect = (self.selection_start_position, self.mouse_position)
            self.add_treeview_icon(rect)
        self.action_status = None

    def on_area_motion_notify_event(self, widget, event):
        if self.action_status is None:
            GtkCairoPlayer.on_area_motion_notify_event(self, widget, event)
        else:
            self.mouse_position = (event.x, event.y)
            self.area.queue_draw()
        # if self.status = rotation:
        #   set something to tell rotate everything
        # elif self.status == selection:
        #    self.mouse_position = x,y

    def draw_selection_box(self, context, region=None):
        context.save()
        if not region:
            context.set_matrix(cairo.Matrix(1, 0, 0, 1, 0, 0))
            #a = context.set_matrix( 1,0,0,1,0,0 )
            #matrix = array([[a[0], a[2], a[4]], [a[1], a[3], a[5]], [0, 0, 1]])
            x0 = self.selection_start_position[0]
            y0 = self.selection_start_position[1]
            x1 = self.mouse_position[0]
            y1 = self.mouse_position[1]
            #x0, y0, nulla = dot(inv(matrix), [x0, y0, 1])
            #x1, y1, nulla = dot(inv(matrix), [x1, y1, 1])
        else:
            context.translate(region.xc, region.yc)
            context.rotate(region.rotate)
            context.translate(-region.xc, -region.yc)
            x0 = region.xc - region.width / 2
            y0 = region.yc - region.height / 2
            x1 = region.xc + region.width / 2
            y1 = region.yc + region.height / 2

        context.set_source_rgb(0, 0xff, 0)
        context.move_to(x0, y0)
        context.line_to(x1, y0)
        context.line_to(x1, y1)
        context.line_to(x0, y1)
        context.line_to(x0, y0)
        context.set_line_width(0.2)
        context.stroke()

        context.rectangle(x0, y0, x1 - x0, y1 - y0)
        context.set_source_rgba(0, 0, 0.5, 0.50)
        context.fill()
        context.restore()

    def render(self, context):
        #   render a transparent box (should indicate the selection)
        # render already existant path points.
        # render an icon for rotating the whole image.
        GtkCairoPlayer.render(self, context)
        for i in xrange(0, len(self.presentation.path)):
            region = self.presentation.get_path_region(i)
            self.draw_selection_box(context, region)
        if self.action_status == "selection":
            self.draw_selection_box(context)
        #self.rotate_icon.render(context, self.area.window.get_size() )

    def add_treeview_icon(self, rect=None):
        ## * trasforma rect in Regione
        win_size = self.area.window.get_size()
        scale = self.current_region.get_window_scale_factor(win_size)
        translate = self.current_region.get_window_fit_vector(win_size)
        translate = translate[0] / 2.0, translate[1] / 2.0
        rect2 = (((rect[0][0] - translate[0]) / scale,
                  (rect[0][1] - translate[1]) / scale),
                 ((rect[1][0] - translate[0]) / scale,
                  (rect[1][1] - translate[1]) / scale))
        xc = (rect2[0][0] + rect2[1][0]) / 2 - self.current_region.width / 2
        yc = (rect2[0][1] + rect2[1][1]) / 2 - self.current_region.height / 2
        r = self.current_region.rotate
        xc1 = math.cos(r) * xc - math.sin(r) * yc
        yc1 = math.sin(r) * xc + math.cos(r) * yc
        region = Region(
                    self.current_region.xc + xc1,
                    self.current_region.yc + yc1,
                    (rect2[1][0] - rect2[0][0]),
                    (rect2[1][1] - rect2[0][1]),
                    self.current_region.rotate)
        #print "Finestra", self.area.window.get_size()
        #print "Current", self.current_region
        #print "Aggiungo", region
        #print
        ## * metti la regione in lista
        self.presentation.path.append(region)
        ## * usa la regione per la pixbuf.
        self.draw_region_on_iconview(region)

    def draw_region_on_iconview(self, region):
        pixbuf_height = abs(int(100 * region.height / region.width))
        pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 100,
                    pixbuf_height)
        pix_data = pixbuf.get_pixels_array()
        pixbuf_size = (pixbuf.get_width(), pixbuf.get_height())
        surface = cairo.ImageSurface.create_for_data(pix_data, cairo.FORMAT_ARGB32,
                     pixbuf_size[0], pixbuf_size[1], pixbuf.get_rowstride())
        cr = cairo.Context(surface)
        region.render(cr, self.presentation, pixbuf_size)

        position = len(self.presentation.path)-1
        self.model.append([position, str(position), pixbuf])


#class RotateIcon(object):
#
#    def __init__(self):
#        self.image = cairo.ImageSurface.create_from_png('./bussola.png')
#        self.image_width = self.image.get_width()
#        self.image_height = self.image.get_height()
#        pass
#
#    def get_bbox(self):
#        pass
#
#    def render(self, context, win_size):
#        a = context.get_matrix()
#        matrix = [[a[0], a[2], a[4]], [a[1], a[3], a[5]], [0, 0, 1]]
#        win_width, win_height = win_size
#        x0 = 0
#        y0 = 0
#        width = self.image_width
#        height = self.image_height
#
#        context.set_source_surface(self.image)
#        context.rectangle(x0, y0, width, height)
#        context.clip()
#        context.paint()
